@@ -4,6 +4,7 @@ from random import sample
 import torch
 import torch.nn as nn
 from astrape.utilities.utils_lightning import set_default_parameters
+from astrape.utilities.utils import rearrange_dims
 from astrape.base.model_base import BaseNet, ConvNet
 from typing import Union, List, Dict, Any, Optional, cast, Tuple
 from astrape.exceptions.exceptions import *
@@ -964,7 +965,8 @@ class _UNet(nn.Module):
         l1_strength : float,
         activation_type : str = "relu",
         negative_slope : Optional[float] = None,
-        model_type : str = "UNet", # don't change this ,
+        model_type : str = "UNet", # don't change this
+        dataformats : Optional[str] = "NCHW", # number of samples, channels, height, width
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ):
         self.n_classes = n_classes
@@ -981,24 +983,13 @@ class _UNet(nn.Module):
         self.sample_weight = sample_weight
         self.activation_type = activation_type 
         self.negative_slope = negative_slope
+        self.dataformats = dataformats
 
         if self.n_layers < 1:
             raise ValueError(f"n_layers = {self.n_layers}, expected: n_layers > 0")
 
-        if isinstance(dims, int):
-            height = width = math.sqrt(dims)
-            if (height != int(height)):
-                raise ValueError("Wrong dimension in input.")
-            dims = (1, height, width)
-        if len(dims) == 2:
-            height = dims[0]
-            width = dims[1]
-            dims = (1, height, width)
-        if len(dims) == 4:
-            dims = dims[1:]
-        if len(dims) > 4:
-            raise ValueError("dimension of the input can't be higher than 4.")
-        self.dims = dims # (in_channels, height, width)
+        
+        self.dims = rearrange_dims(dims=dims, dataformats=dataformats) # (in_channels, height, width)
         self.input_channels = self.dims[0]
         super().__init__()
 
@@ -1028,22 +1019,19 @@ class _UNet(nn.Module):
             xi[-1] = layer(xi[-1], xi[-2 - i])
         return self.layers[-1](xi[-1])
 
-
 class UNet(ConvNet):
     def __init__(
         self,
         dims : Union[int, Tuple[int]],
         n_classes: int,
-        model_type : str = "UNet", # don't change this
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         **hparams : Any
     ):
 
         entire_keys = [
-            'n_classes', 
-            'model_type',
+            'n_classes',
             'dims', 
-            'n_layers', 
+            'n_blocks', 
             'bn',
             'features_start', 
             'bilinear',
@@ -1054,13 +1042,15 @@ class UNet(ConvNet):
             'sample_weight',
             'l1_strength',
             'activation_type',
-            'negative_slope'
+            'negative_slope',
+            'dataformats'
         ]
 
         default_parameter_dict = {
             'n_layers' : 3,
             'features_start' : 64,
-            'bilinear' : False
+            'bilinear' : False,
+            'dataformats' : "NHWC"
         }
          
         # set default for model-specific hyperparameters
@@ -1078,7 +1068,6 @@ class UNet(ConvNet):
             n_classes=n_classes, 
             dims=dims, 
             sample_weight=sample_weight,
-            model_type="UNet",
             **hparams
         )
         # sanity check. Check if right arguments are declared.
@@ -1095,6 +1084,126 @@ class UNet(ConvNet):
     def forward(self, X):
         return self.model.forward(X)
     
+class _PixelCNN(nn.Module):
+    def __init__(
+        self,
+        in_channels : int,
+        dims : Union[int, Tuple[int]], 
+        batch_size : int,
+        optimizer_type : str,
+        lr : float,
+        weight_decay : float,
+        bn : bool,
+        l1_strength : float,
+        activation_type : str = "relu",
+        negative_slope : Optional[float] = None,
+        dataformats : Optional[str] = "NCHW", # number of samples, channels, height, width
+        hidden_channels : int=256,
+        n_blocks : int=5,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
+    ):
+        super().__init__()
+
+        self.in_channels = in_channels
+        self.hidden_channels = hidden_channels
+        self.n_blocks = n_blocks
+        self.batch_size = batch_size
+        self.optimizer_type = optimizer_type
+        self.lr = lr
+        self.weight_decay = weight_decay
+        self.bn = bn
+        self.l1_strength = l1_strength
+        self.activatioin_type = activation_type
+        self.negative_slope = negative_slope
+        self.dataformats = dataformats
+        self.sample_weight = sample_weight
+
+        self.blocks = nn.ModuleList([self.conv_block(in_channels) for _ in range(n_blocks)])
+
+        self.dims = rearrange_dims(dims=dims, dataformats=dataformats)
+    def conv_block(self, in_channels):
+        c1 = nn.Conv2d(in_channels=in_channels, out_channels=self.hidden_channels, kernel_size=(1,1))
+        act1 = nn.ReLU()
+        c2 = nn.Conv2d(in_channels=self.hidden_channels, out_channels=self.hidden_channels, kernel_size=(1,3))
+        pad = nn.ConstantPad2d((0,0,1,0,0,0,0,0),1)
+        c3 = nn.Conv2d(
+            in_channels=self.hidden_channels, out_channels=self.hidden_channels, kernel_size=(2,1), padding=(0,1)
+        )
+        act2 = nn.ReLU()
+        c4 = nn.Conv2d(in_channels=self.hidden_channels, out_channels=in_channels, kernel_size=(1,1))
+        block = nn.Sequential(c1, act1, c2, pad, c3, act2, c4)
+
+        return block
+    
+    def forward(self, x):
+        c = x
+        for conv_block in self.blocks:
+            c = c + conv_block(c)
+        
+        c = nn.ReLU()(c)
+        return c
+
+class PixelCNN(ConvNet):
+    def __init__(
+        self,
+        dims : Union[int, Tuple[int]],
+        n_classes: int,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        **hparams : Any
+    ):
+        entire_keys = [
+            'n_classes',
+            'dims', 
+            'n_blocks', 
+            'hidden_channels',
+            'in_channels',
+            'bn',
+            'batch_size' ,
+            'optimizer_type' ,
+            'lr' ,
+            'weight_decay',
+            'sample_weight',
+            'l1_strength',
+            'activation_type',
+            'negative_slope',
+            'dataformats'
+        ]
+
+        default_parameter_dict = {
+            'n_blocks' : 5,
+            'hidden_channels' : 10,
+            'dataformats' : "NHWC"
+        }
+        
+        for argument, default in default_parameter_dict.items():
+            if argument not in hparams.keys(): 
+                hparams.update({argument : default})
+        #set default for model-agnostic hyperparameters
+        set_default_parameters(hparams)
+        # for some reason, we don't use dropout here
+        hparams.pop('dropout_p')
+        # save information of **hparams to self.hparams.
+        self.save_hyperparameters(*hparams)
+
+        super(PixelCNN, self).__init__(
+            n_classes=n_classes, 
+            dims=dims, 
+            sample_weight=sample_weight,
+            **hparams
+        )
+        # sanity check. Check if right arguments are declared.
+        for key in hparams.keys():
+            if key not in entire_keys:
+                raise ValueError(f'Wrong argument is passed to the model.')
+        self.model = _PixelCNN(
+            n_classes=n_classes, 
+            dims=dims, 
+            sample_weight=sample_weight,
+            **hparams
+        )
+
+    def forward(self, X):
+        return self.model.forward(X)
 
 class _Generator(nn.Module):
     def __init__(
